<?php
/**
 * EDUtwitter: Block for Moodle 2.2.1
 * 
 * 
 * @package  EDUtwitter
 * @author   David Romero Santos <david.romerosantos@alum.uca.es>
 * @license  http://opensource.org/licenses/gpl-license.php GNU Public License
 * @link     https://forja.rediris.es/projects/cusl6-edutweets/
 */

require_once('TwitterAPIExchange.php');
require_once('split_tweet.php');

/**
 * Given a course and a date, send a tweet for every new
 * messages posted in the course since that date
 *
 * @param object $course
 * @param object $block
 * @param bool $viewfullnames capability
 * @param int $timestart
 * @return bool success
 */
function forum_print_recent_activity_twitter($course, $viewfullnames, $timestart, $block) {
    global $CFG, $USER, $DB;

    /** 
    * Variables needed for the Twitter API
    */
    $consumerkey=$block->config->consumerkey;
    $consumersecret=$block->config->consumersecret;
    $oauthtoken=$block->config->oauthtoken;
    $oauthsecret=$block->config->oauthsecret;
    $language=$block->config->language;
    if($block->config->search_term[0] == '#'){
        $search_term=$block->config->search_term;
    } else {
        $search_term='#'.$block->config->search_term;
    }

    $settings = array('oauth_access_token' => $oauthtoken, 'oauth_access_token_secret' => $oauthsecret, 'consumer_key' => $consumerkey, 'consumer_secret' => $consumersecret);
    $url = 'https://api.twitter.com/1.1/statuses/update.json';
    $requestMethod = 'POST';

    /** 
    * Check new posts
    */
    // do not use log table if possible, it may be huge and is expensive to join with other tables
    if (!$posts = $DB->get_records_sql("SELECT p.*, f.type AS forumtype, d.forum, d.groupid,
                                              d.timestart, d.timeend, d.userid AS duserid,
                                              u.firstname, u.lastname, u.email, u.picture
                                         FROM {forum_posts} p
                                              JOIN {forum_discussions} d ON d.id = p.discussion
                                              JOIN {forum} f             ON f.id = d.forum
                                              JOIN {user} u              ON u.id = p.userid
                                        WHERE p.created > ? AND f.course = ?
                                     ORDER BY p.id ASC", array($timestart, $course->id))) { // order by initial posting date
         return false;
    }

    $modinfo =& get_fast_modinfo($course);

    $groupmodes = array();
    $cms    = array();

    $strftimerecent = get_string('strftimerecent');

    $printposts = array();
    foreach ($posts as $post) {
        if (!isset($modinfo->instances['forum'][$post->forum])) {
            // not visible
            continue;
        }
        $cm = $modinfo->instances['forum'][$post->forum];
        if (!$cm->uservisible) {
            continue;
        }
        $context = get_context_instance(CONTEXT_MODULE, $cm->id);

        if (!has_capability('mod/forum:viewdiscussion', $context)) {
            continue;
        }

        if (!empty($CFG->forum_enabletimedposts) and $USER->id != $post->duserid
          and (($post->timestart > 0 and $post->timestart > time()) or ($post->timeend > 0 and $post->timeend < time()))) {
            if (!has_capability('mod/forum:viewhiddentimedposts', $context)) {
                continue;
            }
        }

        $groupmode = groups_get_activity_groupmode($cm, $course);

        if ($groupmode) {
            if ($post->groupid == -1 or $groupmode == VISIBLEGROUPS or has_capability('moodle/site:accessallgroups', $context)) {
                // oki (Open discussions have groupid -1)
            } else {
                // separate mode
                if (isguestuser()) {
                    // shortcut
                    continue;
                }

                if (is_null($modinfo->groups)) {
                    $modinfo->groups = groups_get_user_groups($course->id); // load all my groups and cache it in modinfo
                }

                if (!array_key_exists($post->groupid, $modinfo->groups[0])) {
                    continue;
                }
            }
        }

        $printposts[] = $post;
    }
    unset($posts);

    if (!$printposts) {
        return false;
    }
    /** 
    * Change the tweet if it's a completely new post or it's a response
    */
    foreach ($printposts as $post) {
        if (empty($post->parent)) {
            $url_post=$CFG->wwwroot.'/mod/forum/discuss.php?d='.$post->discussion;
            if($language == '1'){
                $contenido=$search_term.' '.fullname($post, $viewfullnames).' ha puesto el nuevo mensaje "'.$post->subject.'" '.$url_post;
            } else {
                $contenido=$search_term.' '.fullname($post, $viewfullnames).' posted a new message "'.$post->subject.'" '.$url_post;
            }

        } else {
            $url_post=$CFG->wwwroot.'/mod/forum/discuss.php?d='.$post->discussion.'&amp;parent='.$post->parent.'#p'.$post->id;
            if($language == '1'){
                $contenido=$search_term.' '.fullname($post, $viewfullnames).' ha puesto un nuevo mensaje con el asunto "'.$post->subject.'" en '.$url_post;
            } else {
                $contenido=$search_term.' '.fullname($post, $viewfullnames).' posted a new message with the subject "'.$post->subject.'" on '.$url_post;
            }
        }
        $postfields = array('status' => $contenido); 
        $tweet = new TwitterAPIExchange($settings);
        $response = $tweet->buildOauth($url, $requestMethod)
                          ->setPostfields($postfields)
                          ->performRequest();
        $salida = json_decode($response);
        if(!empty($salida->errors)){
            foreach($salida->errors as $error){
                if($error->code == '186'){
                    split_tweet($tweet,$contenido);
                }
            }
        }
    }

    return true;
}

/**
 * Print recent activity from all assignments in a given course
 *
 * @param object $course
 * @param object $block
 * @param bool $viewfullnames capability
 * @param int $timestart
 * @return bool success
 */
function assignment_print_recent_activity_twitter($course, $viewfullnames, $timestart, $block) {
    global $CFG, $USER, $DB, $OUTPUT;

    /** 
    * Variables needed for the Twitter API
    */
    $consumerkey=$block->config->consumerkey;
    $consumersecret=$block->config->consumersecret;
    $oauthtoken=$block->config->oauthtoken;
    $oauthsecret=$block->config->oauthsecret;
    $language=$block->config->language;
    if($block->config->search_term[0] == '#'){
        $search_term=$block->config->search_term;
    } else {
        $search_term='#'.$block->config->search_term;
    }

    $settings = array('oauth_access_token' => $oauthtoken, 'oauth_access_token_secret' => $oauthsecret, 'consumer_key' => $consumerkey, 'consumer_secret' => $consumersecret);
    $url = 'https://api.twitter.com/1.1/statuses/update.json';
    $requestMethod = 'POST';

    // do not use log table if possible, it may be huge
    /** 
    * Check new assignment's submissions
    */
    if (!$submissions = $DB->get_records_sql("SELECT asb.id, asb.timemodified, cm.id AS cmid, asb.userid,
                                                     u.firstname, u.lastname, u.email, u.picture
                                                FROM {assignment_submissions} asb
                                                     JOIN {assignment} a      ON a.id = asb.assignment
                                                     JOIN {course_modules} cm ON cm.instance = a.id
                                                     JOIN {modules} md        ON md.id = cm.module
                                                     JOIN {user} u            ON u.id = asb.userid
                                               WHERE asb.timemodified > ? AND
                                                     a.course = ? AND
                                                     md.name = 'assignment'
                                            ORDER BY asb.timemodified ASC", array($timestart, $course->id))) {
         return false;
    }

    $modinfo =& get_fast_modinfo($course); // reference needed because we might load the groups
    $show    = array();
    $grader  = array();

    foreach($submissions as $submission) {
        if (!array_key_exists($submission->cmid, $modinfo->cms)) {
            continue;
        }
        $cm = $modinfo->cms[$submission->cmid];
        if (!$cm->uservisible) {
            continue;
        }
        if ($submission->userid == $USER->id) {
            $show[] = $submission;
            continue;
        }

        // the act of sumbitting of assignment may be considered private - only graders will see it if specified
        if (empty($CFG->assignment_showrecentsubmissions)) {
            if (!array_key_exists($cm->id, $grader)) {
                $grader[$cm->id] = has_capability('moodle/grade:viewall', get_context_instance(CONTEXT_MODULE, $cm->id));
            }
            if (!$grader[$cm->id]) {
                continue;
            }
        }

        $groupmode = groups_get_activity_groupmode($cm, $course);

        if ($groupmode == SEPARATEGROUPS and !has_capability('moodle/site:accessallgroups', get_context_instance(CONTEXT_MODULE, $cm->id))) {
            if (isguestuser()) {
                // shortcut - guest user does not belong into any group
                continue;
            }

            if (is_null($modinfo->groups)) {
                $modinfo->groups = groups_get_user_groups($course->id); // load all my groups and cache it in modinfo
            }

            // this will be slow - show only users that share group with me in this cm
            if (empty($modinfo->groups[$cm->id])) {
                continue;
            }
            $usersgroups =  groups_get_all_groups($course->id, $submission->userid, $cm->groupingid);
            if (is_array($usersgroups)) {
                $usersgroups = array_keys($usersgroups);
                $intersect = array_intersect($usersgroups, $modinfo->groups[$cm->id]);
                if (empty($intersect)) {
                    continue;
                }
            }
        }
        $show[] = $submission;
    }

    if (empty($show)) {
        return false;
    }
    
    /** 
    * Publish submissions on Twitter
    */
    foreach ($show as $submission) {
        $cm = $modinfo->cms[$submission->cmid];
        $link = $CFG->wwwroot.'/mod/assignment/view.php?id='.$cm->id;
        if($language == '1'){
            $contenido = $search_term.' '.$USER->firstname.' '.$USER->lastname.' ha realizado "'.$cm->name.'" a las '.date("H:i \d\e\l \d\i\a d \d\e\l m",$submission->timemodified).' '.$link;
        } else {
            $contenido = $search_term.' '.$USER->firstname.' '.$USER->lastname.' has sent "'.$cm->name.'" on '.date("F j, H:m",$submission->timemodified).' '.$link;
        }
        $postfields = array('status' => $contenido); 
        $tweet = new TwitterAPIExchange($settings);
        $response = $tweet->buildOauth($url, $requestMethod)
                          ->setPostfields($postfields)
                          ->performRequest();
        $salida = json_decode($response);
        if(!empty($salida->errors)){
            foreach($salida->errors as $error){
                if($error->code == '186'){
                    split_tweet($tweet,$contenido);
                }
            }
        }
    }

    return true;
}

/**
 * Send tweets with the recent activity
 *
 * @param object $course
 * @param object $block
 */
function recent_activity($course, $block) {
    global $CFG, $USER, $SESSION, $DB, $OUTPUT;

    /** 
    * Get last cron execution
    */
    $record = $DB->get_record('block_edutwitter_params', array('courseid'=>$course->id));
    $timestart=$record->time_recent;

    $context = get_context_instance(CONTEXT_COURSE, $course->id);

    $viewfullnames = has_capability('moodle/site:viewfullnames', $context);

    /** 
    * Variables needed for send tweets
    */
    $consumerkey=$block->config->consumerkey;
    $consumersecret=$block->config->consumersecret;
    $oauthtoken=$block->config->oauthtoken;
    $oauthsecret=$block->config->oauthsecret;
    $language=$block->config->language;
    if($block->config->search_term[0] == '#'){
        $search_term=$block->config->search_term;
    } else {
        $search_term='#'.$block->config->search_term;
    }

    $settings = array('oauth_access_token' => $oauthtoken, 'oauth_access_token_secret' => $oauthsecret, 'consumer_key' => $consumerkey, 'consumer_secret' => $consumersecret);
    $url = 'https://api.twitter.com/1.1/statuses/update.json';
    $requestMethod = 'POST';
    
    /** 
    * Firstly, have there been any new enrolments?
    */
    $users = get_recent_enrolments($course->id, $timestart);
    if ($users) {
        foreach ($users as $user) {
            $fullname = fullname($user, $viewfullnames);
            $url_post=$CFG->wwwroot.'/user/view.php?id='.$user->id.'->id&amp;course='.$course->id.'/';
            if($language == '1'){
                $contenido = $search_term.' Nuevo estudiante en el curso: '.$fullname.' '.$url_post;
            } else {
                $contenido = $search_term.' New student in the course: '.$fullname.' '.$url_post;
            }
            $postfields = array('status' => $contenido); 
            $tweet = new TwitterAPIExchange($settings);
            $response = $tweet->buildOauth($url, $requestMethod)
                              ->setPostfields($postfields)
                              ->performRequest();
            $salida = json_decode($response);
            if(!empty($salida->errors)){
                foreach($salida->errors as $error){
                    if($error->code == '186'){
                        split_tweet($tweet,$contenido);
                    }
                }
            }
        }
    }

    /** 
    * Next, have there been any modifications to the course structure?
    */

    $modinfo =& get_fast_modinfo($course);

    $changelist = array();

    $logs = $DB->get_records_select('log', "time > ? AND course = ? AND
                                            module = 'course' AND
                                            (action = 'add mod' OR action = 'update mod' OR action = 'delete mod')",
                                    array($timestart, $course->id), "id ASC");
   
    /** 
    * Get the activities previously added but hidden and add them to $logs
    */

    $old_logs = $DB->get_records('block_edutwitter_adds',array('courseid' => $course->id));

    foreach($old_logs as $old_log){
        $hidden = $DB->get_record('log',array('course' => $course->id, 'cmid' => $old_log->cmid));
        $id = $hidden->id-1;
        $hidden_true = $DB->get_record('log',array('id' => $id));
        $logs[] = $hidden_true;
    }

    /** 
    * Look last adds, updates and deletes of activities
    */

    if ($logs) {
        $actions  = array('add mod', 'update mod', 'delete mod');
        $newgones = array(); // added and later deleted items

        foreach ($logs as $key => $log) {
            if (!in_array($log->action, $actions)) {
                continue;
            }
            $info = explode(' ', $log->info);

            // note: in most cases I replaced hardcoding of label with use of
            // $cm->has_view() but it was not possible to do this here because
            // we don't necessarily have the $cm for it
            if ($info[0] == 'label') {     // Labels are ignored in recent activity
                continue;
            }

            if (count($info) != 2) {
                debugging("Incorrect log entry info: id = ".$log->id, DEBUG_DEVELOPER);
                continue;
            }

            $modname    = $info[0];
            $instanceid = $info[1];
            
            if ($log->action == 'delete mod') {
                // unfortunately we do not know if the mod was visible
                if (!array_key_exists($log->info, $newgones)) {
                    if($language == '1'){
                        $strdeleted = 'Eliminado';
                    } else {
                        $strdeleted = 'Deleted';
                    }
                }
            } else {
                if (!isset($modinfo->instances[$modname][$instanceid])) {
                    if ($log->action == 'add mod') {
                        // do not display added and later deleted activities
                        $newgones[$log->info] = true;
                    }
                    continue;
                }
                $cm = $modinfo->instances[$modname][$instanceid];
                if (!$cm->uservisible) {
                    continue;
                }

                if ($log->action == 'add mod') {
                    if($cm->visible == '1'){
                        if($language == '1'){
                            $stradded = 'Agregado';
                        } else {
                            $stradded = 'Added';
                        }
                        $url_content = $CFG->wwwroot.'/mod/'.$cm->modname.'/view.php?id='.$cm->id;
                        $content = $search_term.' '.$stradded.' "'.$cm->name.'" '.$url_content;
                        $changelist[$log->info] = array('operation' => 'add', 'text' => $content);
                        if($DB->record_exists('block_edutwitter_adds', array('courseid'=>$course->id, 'cmid'=>$cm->id))){
                            $DB->delete_records('block_edutwitter_adds',array('courseid'=>$course->id, 'cmid'=>$cm->id));
                        }
                    } else {
                        if(!$DB->record_exists('block_edutwitter_adds', array('courseid'=>$course->id, 'cmid'=>$cm->id))){
                            $record_log = new stdClass();
                            $record_log->courseid = $course->id;
                            $record_log->cmid = $cm->id;
                            $DB->insert_record('block_edutwitter_adds',$record_log);
                        }         
                    }
                } else if ($log->action == 'update mod' and empty($changelist[$log->info])) {
                    if($cm->visible == '1'){
                        if($language == '1'){
                            $strupdated = 'Actualizado';
                        } else {
                            $strupdated = 'Updated';
                        }
                        $url_content = $CFG->wwwroot.'/mod/'.$cm->modname.'/view.php?id='.$cm->id;
                        $content = $search_term.' '.$strupdated.' "'.$cm->name.'" '.$url_content;
                        $changelist[$log->info] = array('operation' => 'update', 'text' => $content);
                    }
                }
            }
        }
    }

    /** 
    * Send tweets with changes
    */
    if (!empty($changelist)) {
        $content = true;
        foreach ($changelist as $changeinfo => $change) {
            $contenido = $change['text'];
            $postfields = array('status' => $contenido); 
            $tweet = new TwitterAPIExchange($settings);
            $response = $tweet->buildOauth($url, $requestMethod)
                              ->setPostfields($postfields)
                              ->performRequest();
            $salida = json_decode($response);
            if(!empty($salida->errors)){
                foreach($salida->errors as $error){
                    if($error->code == '186'){
                        split_tweet($tweet,$contenido);
                    }
                }
            }
        }
    }


    /** 
    * Now display new things from each module
    */

    $modinfo =& get_fast_modinfo($course);

    $usedmodules = array();
    foreach($modinfo->cms as $cm) {
        if (isset($usedmodules[$cm->modname])) {
            continue;
        }
        if (!$cm->uservisible) {
            continue;
        }
        $usedmodules[$cm->modname] = $cm->modname;
    }

    $content = false;    
    
    foreach ($usedmodules as $modname) {      // Each module gets it's own logs and prints them
        if (file_exists($CFG->dirroot.'/mod/'.$modname.'/lib.php')) {
            include_once($CFG->dirroot.'/mod/'.$modname.'/lib.php');
            $print_recent_activity = $modname.'_print_recent_activity_twitter';
            if (function_exists($print_recent_activity)) {
                // NOTE: original $isteacher (second parameter below) was replaced with $viewfullnames!
                $content = $print_recent_activity($course,$viewfullnames, $timestart,$block) || $content;
            }
        } else {
            debugging("Missing lib.php in lib/{$modname} - please reinstall files or uninstall the module");
        }
    }
 
    /** 
    * Keep time for next execution
    */
    $record->time_recent = time();
    $DB->update_record('block_edutwitter_params', $record);
}
