<?php
/**
 * EDUtwitter: Block for Moodle 2.2.1
 * 
 * 
 * @package  EDUtwitter
 * @author   David Romero Santos <david.romerosantos@alum.uca.es>
 * @license  http://opensource.org/licenses/gpl-license.php GNU Public License
 * @link     https://forja.rediris.es/projects/cusl6-edutweets/
 */

require_once('../../config.php');
require_once('statistics_form.php');
require_once('TwitterAPIExchange.php');

global $DB, $OUTPUT, $PAGE;

//Check for all required variables.
$courseid = required_param('courseid', PARAM_INT);
$blockid = required_param('blockid', PARAM_INT);
$consumerkey = required_param('consumerkey', PARAM_MULTILANG);
$consumersecret = required_param('consumersecret', PARAM_MULTILANG);
$oauthtoken = required_param('oauthtoken', PARAM_MULTILANG);
$oauthsecret = required_param('oauthsecret', PARAM_MULTILANG);
$search_term = required_param('search_term', PARAM_MULTILANG);

//Next look for optional variables.
$id = optional_param('id', 0, PARAM_INT);

if (!$course = $DB->get_record('course', array('id' => $courseid))){
    print_error('invalidcourse', 'block_edutwitter', $courseid);
}

require_login($course);

$PAGE->set_url('/blocks/edutwitter/view_statistics.php', array('id' => $courseid));
$PAGE->set_pagelayout('standard');
$PAGE->set_heading(get_string('edithtml', 'block_edutwitter'));

$settingsnode = $PAGE->settingsnav->add(get_string('edutwittersettings', 'block_edutwitter'));
$editurl = new moodle_url('/blocks/edutwitter/view_statistics.php', array('id' => $id, 'courseid' => $courseid, 'blockid' => $blockid));
$editnode = $settingsnode->add(get_string('editpage_est', 'block_edutwitter'), $editurl);
$editnode->make_active();

$statistics = new statistics_form();

$toform['blockid'] = $blockid;
$toform['courseid'] = $courseid;
$toform['consumerkey'] = $consumerkey;
$toform['consumersecret'] = $consumersecret;
$toform['oauthtoken'] = $oauthtoken;
$toform['oauthsecret'] = $oauthsecret;
$toform['search_term'] = $search_term;
$statistics->set_data($toform);

if($statistics->is_cancelled()) {
    //Cancelled forms redirect to the course main page
    $courseurl = new moodle_url('/course/view.php', array('id' => $courseid));
    redirect($courseurl);
} else if ($fromform = $statistics->get_data()) {
    $total = 0;
    $records = $DB->get_recordset('block_edutwitter_users', array('courseid' => $courseid), 'userlast');
    /**
    * Count all the records with the course's hashtag
    */
    foreach($records as $record){
        $total = $total+$record->numbertweets;
    }
    echo "<table border=1 cellpading=4 cellspacing=0>";
    echo "<tr>
            <th colspan=5 align='center'> ".get_string('url_statistics', 'block_edutwitter')." </th>
          </tr>
          <tr>
            <th> ".get_string('surname', 'block_edutwitter')." </th><th> ".get_string('name', 'block_edutwitter')." </th><th> ".get_string('pagetitle_submit', 'block_edutwitter')." </th>
            <th> ".get_string('number_tweets', 'block_edutwitter')." </th><th> ".get_string('percentage', 'block_edutwitter')." </th>
          </tr>";
    $records = $DB->get_recordset('block_edutwitter_users', array('courseid' => $courseid), 'userlast'); 
    foreach($records as $record){
        /**
        * Show statistics table for each user
        */
        if($total != 0){
            $porcentaje = round(($record->numbertweets/$total)*100,2).'%';
        } else {
            $porcentaje = '0%';
        }
        $usuario = '@'.$record->twitteruser;
        echo "<tr>
                <td> $record->userlast </td>
                <td> $record->username </td>
                <td align='center'> $usuario </td>
                <td align='right'> $record->numbertweets </td>
                <td align='right'> $porcentaje </td>
              </tr>";
    }
    echo "</table>";
    $records->close();
    if($fromform->clean_statistics){
        $records = $DB->get_recordset('block_edutwitter_users', array('courseid' => $courseid), 'userlast');
        foreach($records as $record){
            $record->numbertweets = 0;
            $DB->update_record('block_edutwitter_users', $record);
        }
        $records->close();
    }    
} else {
    // form didn't validate or this is the first display
    $site = get_site();
    echo $OUTPUT->header();
    $statistics->display();
    echo $OUTPUT->footer();
}

?>
